import logging

from datetime import timezone
import dateutil.parser
from tzlocal import get_localzone


_LOGGER = logging.getLogger(__name__)


class EcoNetVacation(object):
    """
    Represents an EcoNet vacation.
    """

    def __init__(self, vacation_as_json, api_interface):
        self.api_interface = api_interface
        self.vacation_json = vacation_as_json

    @property
    def id(self):
        return self.vacation_json.get("id")

    @property
    def start_date(self):
        date_string = self.vacation_json.get("startDate")
        utc_date = dateutil.parser.parse(date_string)
        local_tz = get_localzone()
        return utc_date.replace(tzinfo=timezone.utc).astimezone(local_tz)

    @property
    def end_date(self):
        date_string = self.vacation_json.get("endDate")
        utc_date = dateutil.parser.parse(date_string)
        local_tz = get_localzone()
        return utc_date.replace(tzinfo=timezone.utc).astimezone(local_tz)

    @property
    def location_id(self):
        return self.vacation_json["location"]["id"]

    @property
    def equipment_ids(self):
        _ids = []
        equipment_ids = self.vacation_json["participatingEquipment"]
        for equipment_id in equipment_ids:
            _ids.append(equipment_id.get("id"))
        return _ids

    def delete(self):
        self.api_interface.delete_vacation(str(self.id))



import logging
import json

import requests

from .equipment.water_heater import EcoNetWaterHeater
from .vacation import EcoNetVacation

BASE_URL = "https://econet-api.rheemcert.com"
LOCATIONS_URL = BASE_URL + "/locations"
DEVICE_URL = BASE_URL + "/equipment/%s"
MODES_URL = DEVICE_URL + "/modes"
USAGE_URL = DEVICE_URL + "/usage"
VACATIONS_URL = BASE_URL + "/vacations"
HEADERS = {"Authorization": "Bearer %s", "Content-Type": "application/json"}
BASIC_HEADERS = {"Authorization": "Basic Y29tLnJoZWVtLmVjb25ldF9hcGk6c3RhYmxla2VybmVs"}
USERNAME = None
PASSWORD = None

_LOGGER = logging.getLogger(__name__)


class EcoNetApiInterface(object):
    """
    API interface object.
    """

    def __init__(self, email, password):
        """
        Create the EcoNet API interface object.
        Args:
            email (str): EcoNet account email address.
            password (str): EcoNet account password.
        """
        self.email = email
        self.password = password
        self.token = None
        self.refresh_token = None
        self.last_api_call = None
        self.state = []
        # get a token
        self.authenticated = self._authenticate()

    @staticmethod
    def set_state(_id, body):
        """
        Set a devices state.
        """
        url = DEVICE_URL % _id
        if "mode" in body:
            url = MODES_URL % _id
        arequest = requests.put(url, headers=HEADERS, data=json.dumps(body))
        status_code = str(arequest.status_code)
        if status_code != '202':
            _LOGGER.error("State not accepted. " + status_code)
            return False

    @staticmethod
    def get_modes(_id):
        """
        Pull a water heater's modes from the API.
        """
        url = MODES_URL % _id
        arequest = requests.get(url, headers=HEADERS)
        status_code = str(arequest.status_code)
        if status_code == '401':
            _LOGGER.error("Token expired.")
            return False
        return arequest.json()

    @staticmethod
    def get_usage(_id):
        """
        Pull a water heater's usage report from the API.
        """
        url = USAGE_URL % _id
        arequest = requests.get(url, headers=HEADERS)
        status_code = str(arequest.status_code)
        if status_code == '401':
            _LOGGER.error("Token expired.")
            return False
        try:
            return arequest.json()
        except ValueError:
            _LOGGER.info("Failed to get usage. Not supported by unit?")
            return None

    @staticmethod
    def get_device(_id):
        """
        Pull a device from the API.
        """
        url = DEVICE_URL % _id
        arequest = requests.get(url, headers=HEADERS)
        status_code = str(arequest.status_code)
        if status_code == '401':
            _LOGGER.error("Token expired.")
            return False
        return arequest.json()

    @staticmethod
    def get_locations():
        """
        Pull the accounts locations.
        """
        arequest = requests.get(LOCATIONS_URL, headers=HEADERS)
        status_code = str(arequest.status_code)
        if status_code == '401':
            _LOGGER.error("Token expired.")
            return False
        return arequest.json()

    @staticmethod
    def get_vacations():
        """
        Pull the accounts vacations.
        """
        arequest = requests.get(VACATIONS_URL, headers=HEADERS)
        status_code = str(arequest.status_code)
        if status_code == '401':
            _LOGGER.error("Token expired.")
            return False
        return arequest.json()

    @staticmethod
    def create_vacation(body):
        """
        Create a vacation.
        """
        arequest = requests.post(VACATIONS_URL, headers=HEADERS, data=json.dumps(body))
        status_code = str(arequest.status_code)
        if status_code != '200':
            _LOGGER.error("Failed to create vacation. " + status_code)
            _LOGGER.error(arequest.json())
            return False
        return arequest.json()

    @staticmethod
    def delete_vacation(_id):
        """
        Delete a vacation by ID.
        """
        arequest = requests.delete(VACATIONS_URL + "/" + _id, headers=HEADERS)
        status_code = str(arequest.status_code)
        if status_code != '202':
            _LOGGER.error("Failed to delete vacation. " + status_code)
            return False
        return True

    def _authenticate(self):
        """
        Authenticate with the API and return an authentication token.
        """
        auth_url = BASE_URL + "/auth/token"
        payload = {'username': self.email, 'password': self.password, 'grant_type': 'password'}
        arequest = requests.post(auth_url, data=payload, headers=BASIC_HEADERS)
        status = arequest.status_code
        if status != 200:
            _LOGGER.error("Authentication request failed, please check credintials. " + str(status))
            return False
        response = arequest.json()
        _LOGGER.debug(str(response))
        self.token = response.get("access_token")
        self.refresh_token = response.get("refresh_token")
        _auth = HEADERS.get("Authorization")
        _auth = _auth % self.token
        HEADERS["Authorization"] = _auth
        _LOGGER.info("Authentication was successful, token set.")
        return True


# pylint: disable=too-few-public-methods
class PyEcoNet(object):
    """
    Object used to interact with this library.
    """

    def __init__(self, username, password):
        self.api_interface = EcoNetApiInterface(username, password)
        self.connected = self.api_interface.authenticated
        self.locations = self.api_interface.get_locations()

    def get_water_heaters(self):
        """
        Return a list of water heater devices.

        Parses the response from the locations endpoint in to a pyeconet.WaterHeater.
        """
        water_heaters = []
        for location in self.locations:
            _location_id = location.get("id")
            for device in location.get("equipment"):
                if device.get("type") == "Water Heater":
                    water_heater_modes = self.api_interface.get_modes(device.get("id"))
                    water_heater_usage = self.api_interface.get_usage(device.get("id"))
                    water_heater = self.api_interface.get_device(device.get("id"))
                    vacations = self.api_interface.get_vacations()
                    device_vacations = []
                    for vacation in vacations:
                        for equipment in vacation.get("participatingEquipment"):
                            if equipment.get("id") == water_heater.get("id"):
                                device_vacations.append(EcoNetVacation(vacation, self.api_interface))
                    water_heaters.append(EcoNetWaterHeater(water_heater, water_heater_modes, water_heater_usage,
                                                           _location_id,
                                                           device_vacations,
                                                           self.api_interface))
        return water_heaters



import logging
import json
import time

from ..vacation import EcoNetVacation

_LOGGER = logging.getLogger(__name__)


class EcoNetWaterHeater(object):
    """
    Represents an EcoNet water heater.
    This is a combination of five API endpoints.
    /locations/
    /equipment/{ID}
    /equipment/{ID}/modes
    /equipment/{ID}/usage
    /vacations/
    """

    def __init__(self, device_as_json, device_modes_as_json, device_usage_json, location_id, vacations, api_interface):
        self.api_interface = api_interface
        self.json_state = device_as_json
        self._usage = device_usage_json
        self._modes = []
        self.location_id = location_id
        self.vacations = vacations
        self._last_update = time.time()
        for mode in device_modes_as_json:
            self._modes.append(mode.get("name"))

    @property
    def name(self):
        return self.json_state.get('name')

    @property
    def id(self):
        return self.json_state.get('id')

    @property
    def set_point(self):
        return self.json_state.get("setPoint")

    @property
    def min_set_point(self):
        return self.json_state.get("minSetPoint")

    @property
    def max_set_point(self):
        return self.json_state.get("maxSetPoint")

    @property
    def is_on_vacation(self):
        return self.json_state.get("isOnVacation")

    @property
    def is_connected(self):
        return self.json_state.get("isConnected")

    @property
    def is_enabled(self):
        return self.json_state.get("isEnabled")

    @property
    def in_use(self):
        return self.json_state.get("inUse")

    @property
    def mode(self):
        return self.json_state.get("mode")
    
    @property
    def upper_temp(self):
        return self.json_state.get('upperTemp')

    @property
    def lower_temp(self):
        return self.json_state.get('lowerTemp')

    @property
    def ambient_temp(self):
        return self.json_state.get('ambientTemp')

    @property
    def supported_modes(self):
        return self._modes

    @property
    def usage_unit(self):
        if self._usage and self._usage.get('energyUsage'):
            return self._usage['energyUsage'].get('unit')

    @property
    def total_usage_for_today(self):
        if self._usage and self._usage.get('energyUsage'):
            hours = self._usage['energyUsage'].get('hours', {})
            return sum((usage for usage in hours.values()))

    def get_vacations(self):
        return self.vacations

    def dump_usage_json(self):
        if self._usage:
            return json.dumps(self._usage, indent=4, sort_keys=True)

    def update_state(self):
        now = time.time()
        # Only call the API once to obtain all devices
        if now - self._last_update > 300:
            _LOGGER.info("Calling API to get updated state.")
            device_state = self.api_interface.get_device(self.id)
            if device_state:
                _LOGGER.debug(device_state)
                self.json_state = device_state
            usage = self.api_interface.get_usage(self.id)
            if usage:
                _LOGGER.debug(usage)
                self._usage = usage
            vacations = self.api_interface.get_vacations()
            if vacations:
                self.vacations = []
                for vacation in vacations:
                    for equipment in vacation.get("participatingEquipment"):
                        if equipment.get("id") == self.id:
                            self.vacations.append(EcoNetVacation(vacation, self.api_interface))
            self._last_update = now

    def set_target_set_point(self, temp):
        if self.min_set_point < temp < self.max_set_point:
            self.api_interface.set_state(self.id, {"setPoint": temp})
            self._last_update = 0
        else:
            error = "Invalid set point. Must be < %s and > %s" % self.max_set_point, self.min_set_point
            _LOGGER.error(error)

    def set_vacation_mode(self, start_date, end_date):
        _location_id = self.location_id
        _id = self.id

        start_date_string = start_date.strftime("%Y-%m-%dT%H:%M:%S")
        end_date_string = end_date.strftime("%Y-%m-%dT%H:%M:%S")

        body = {"startDate": start_date_string, "endDate": end_date_string,
                "location": { "id": _location_id },
                "participatingEquipment": [{"id": _id}]}
        self.api_interface.create_vacation(body)
        self._last_update = 0

    def set_mode(self, mode):
        if mode in self._modes:
            self.api_interface.set_state(self.id, {"mode": mode})
            self._last_update = 0
        else:
            _LOGGER.error("Invalid mode: " + str(mode))
